unit Usemodul;
{
Purpose : Basically this unit is responsible for managing the module list
          and any cross references
Author  : Chris R  August 1996
Modified:
}

interface
uses classes,stdctrls,sysutils;

type
{}
   TXref = class
     CallingModuleNo :integer;
     CalledModuleNo  :integer;
   end;

{}
   TXrefList = class(TList)
   private
     function  GetXref(index:integer):TXref;
     procedure SetXref(index:integer;Xref:TXref);
   public
     destructor destroy; override;
     procedure FreeAndClear;
     property items[index:integer]:TXref read getXref write SetXref;
   end;

   TModuleName = string[30];
   TmoduleState = (msUnProcessed,msNotFound,msDone,msLibrary);
{}
   Tmodule = class
     private
       FModuleName :TModuleName;
       FDirName   :string[80];
       FLineCount :integer;
       FModuleState : TModuleState;
       FCalledByCount :integer;
     public
     constructor create(FileName:TFileName);
     function FullPathName:string;
     property ModuleName:TModuleName read FmoduleName;
     property State:TModuleState read FModuleState write FModuleState;
     property LineCount:integer read FLineCount;
     property CalledByCount:integer read FCalledByCount;
     end;

{}
   TModuleList = class(TList)
     private
       FMainFile :TFileName;
       FSearchPath :TStringList;
       FLibraryINI :TFileName;
       XrefList :TXrefList;
       function GetModule(Index:integer):TModule;
       procedure SetModule(Index:integer;ThisModule:TModule);
       procedure SearchOnPath(var ThisFileName:TFileName);
     public
       constructor create;
       destructor destroy; override;
       procedure fillfromScreen(MainFile:TFileName;SearchPath:string);
       procedure PopulateList(Monitor:TLabel);
       function indexof(ThisModuleName:TModuleName):integer;
       procedure GetWhereUsed(ModuleName:string;NameList:Tstrings);
       procedure SaveToDisk(const fname:string);
       property Items[Index:integer]:TModule read GetModule write SetModule;
   end;

implementation
uses dialogs,forms,useparse;

{}
constructor TModule.create(FileName:TFileName);
begin
   if copy(FileName,1,1)='''' then
      delete(FileName,1,1);
   if copy(FileName,length(FileName),1)='''' then
      delete(FileName,length(FileName),1);
   FModuleName := uppercase(extractFileName(FileName));
   FDirName    := extractFilePath(FileName);
   FLineCount  := 0;
   FModuleState := msUnprocessed;
   if not fileexists(FullPathName) then
     FModuleState := msNotFound;
   FCalledByCount := 0;

end;

{}
function TModule.FullPathName:string;
begin
  result := FDirName+FModuleName;
end;

{}
constructor TModuleList.create;
begin
  XrefList := TxrefList.create;
  FSearchPath := Tstringlist.create;
end;

{}
destructor TModuleList.destroy;
var
   i :integer;
begin
  XrefList.free;
  FSearchPath.free;
  for i:= 0 to count-1 do
    items[i].free;
  inherited destroy;
end;

{}
procedure TModuleList.fillfromScreen(MainFile:TFileName;SearchPath:string);
var
   WorkingList :Tstringlist;
   MainFileFound   :boolean;
   NewModule   :TModule;
   i :integer;
   LibraryIni:TFileName;
   nextsemi :integer;
begin
  XrefList.freeandClear;
  Clear;
  FMainFile := Mainfile;
  MainFileFound :=fileexists(Mainfile);
  {if its not found try and add a .pas or a .dpr}
  if (not MainFileFound) and (pos('.',MainFile)=0) then
    if fileexists(Mainfile+'.dpr') then
       MainFile := MainFile+'.dpr'
    else
    if fileexists(Mainfile+'.pas') then
       MainFile := MainFile+'.pas';
  MainFileFound :=fileexists(Mainfile);
  if not MainFileFound then
     raise exception.create(MainFile+' not found');
  NewModule := TModule.create(MainFile);
  add(NewModule); {to the module list }
  NewModule := nil;
  {Now lets process the library file }
  WorkingList := Tstringlist.create;
  LibraryIni := extractFilePath(Application.exename)+'VCLLIST.TXT';
  with WorkingList do
  begin
    loadfromfile(LibraryIni);
    for i:= 0 to count-1 do
    begin
       if pos('.',WorkingList[i])=0 then
         WorkingList[i] := WorkingList[i]+'.pas';
       NewModule := TModule.create(WorkingList[i]);
       NewModule.state := msLibrary;
       self.add(NewModule);   {add to Modulelist not stringlist }
       NewModule := nil;
    end;
    clear;
  end;
  WorkingList.free;
  {Now process Search Path}
  FSearchPath.clear;
  SearchPath := SearchPath+';';
  nextSemi := pos(';',SearchPath);
  while nextSemi>1 do
  begin
     FSearchPath.add(copy(SearchPath,1,NextSemi-1));
     system.delete(SearchPath,1,NextSemi);
     nextSemi := pos(';',SearchPath);
  end;
  {Place a trailing \ for tidiness }
  for i:= 0 to FSearchPath.count-1 do
     if not (copy(FSearchPath[i],length(FSearchPath[i]),1)='\') then
        FSearchPath[i] := FSearchPath[i]+'\';
end; {of create}

{}
procedure TModuleList.GetWhereUsed(ModuleName:string;NameList:Tstrings);
var
   ThisModuleNo :integer;
   i :integer;
begin
   with NameList do
   begin
     clear;
     {Remove the first columns of the entry }
     while pos(' ',ModuleName)>0 do
       system.delete(ModuleName,1,pos(' ',ModuleName));
     ModuleName := extractFileName(ModuleName);
     ThisModuleNo := self.indexof(ModuleName);
     if ThisModuleNo<0 then
       add(ModuleName+' not found')
     else
       for i:=0 to XrefList.count-1 do
         if XrefList.items[i].CalledModuleNo = ThisModuleNo then
         begin
            add(items[XrefList.items[i].CallingModuleNo].ModuleName);
         end
         else
           Application.ProcessMessages;
   end;
end;

{}
function  TModuleList.indexof(ThisModuleName:TModuleName):integer;
var
  i :integer;
begin
  result := -1;
  if pos('.',ThisModuleName)=0 then
    ThisModuleName := ThisModuleName+'.pas';
  ThisModuleName := uppercase(ThisModuleName);
  for i := count-1 downto 0 do
    if items[i].FModuleName = ThisModuleName then
    begin
       result := i;
       exit;
    end;
end; {of indexof }

{}
function  TModuleList.GetModule(Index:integer):TModule;
begin
   result := inherited items[index];
end;

{}
procedure TModuleList.PopulateList(Monitor:TLabel);
var
  ModuleNo :integer;
  CalledModuleNo :integer;
  ThisFileName :TFileName;
  CalledModuleNames :TstringList;
  Parser   :TUsesParser;
  NewModule :TModule;
  i :integer;
  FileOut :textfile;
  Xref :TXref;
begin
  CalledModuleNames := TstringList.create;
  Parser := TUsesParser.create;
  assignfile(FileOut,'usedxref.txt');
  rewrite(FileOut);
  try
    ModuleNo := 0;
    while ModuleNo < Count do
    begin
      Monitor.caption := 'Processing '+inttostr(ModuleNo+1)+' of '+inttostr(count);
      if (items[ModuleNo].state = msUnprocessed) and (fileexists(items[ModuleNo].fullpathname)) then
      begin
        Parser.FileName := items[ModuleNo].fullpathname;
        while Parser.LocateUsesStatement do
        begin
          Parser.ReadUsesStatement(CalledModuleNames);
          for i := 0 to CalledModuleNames.count-1 do
          begin
            ThisFileName := CalledModuleNames[i];
            CalledModuleNo := indexof(ThisFileName);
            if CalledModuleNo<0 then
            begin
              SearchOnPath(ThisFileName);  {inserts directory where found}
              NewModule := TModule.create(ThisFileName);
              self.add(NewModule);
              CalledModuleNo := count-1;
              writeln(FileOut,format('"%s","%s","%s","%s"',
                 [extractfilename(items[ModuleNo].fullpathname),
                  extractfilepath(items[ModuleNo].fullpathname),
                  extractfilename(NewModule.fullpathName),
                  extractfilepath(NewModule.fullpathName)]));
              NewModule := nil;
            end; {module not already loaded }
            inc(items[CalledModuleNo].FcalledbyCount);
            Xref := TXref.create;
            Xref.CallingModuleNo := ModuleNo;
            Xref.CalledModuleNo  := CalledModuleNo;
            XrefList.add(Xref);
            Xref := nil;
          end; {of this to-module refernece }
        end; {of this uses statement }
        items[ModuleNo].FLineCount := Parser.CurrentLineNo;
      end;  {of this from-module }
      ModuleNo := ModuleNo+1;
    end; {of while module no }
    Monitor.caption := '';
  except
    on e:exception do
      raise exception.create(e.message+' on module '+inttostr(moduleno));
  end;
  closefile(FileOut);
  CalledModuleNames.free;
  Parser.free;
end; {of populate List}

{}
procedure TModuleList.SetModule(Index:integer;ThisModule:TModule);
begin
   inherited items[index] := ThisModule;
end;

{}
{ change the parameter if the file cannot be found and the path is not explicit}
procedure TModuleList.SearchOnPath(var ThisFileName:TFileName);
var
  FileNameOnly :TFileName;
  PathNameOnly  :TFileName;
  i :integer;
begin
  if fileexists(ThisFileName) then
    exit;
  if pos('.',ThisFileName)=0 then
  begin
    ThisFileName := ThisFileName+'.pas';
    if fileexists(ThisFileName) then
      exit;
  end;
  if extractFilepath(ThisFileName)>'' then
    exit;
  FileNameOnly := extractFileName(ThisFileName);
  for i := 0 to FSearchPath.count-1 do
    if fileexists(FSearchPath[i]+FileNameOnly) then
    begin
       ThisFileName := FSearchPath[i]+FileNameOnly;
       exit;
    end;
end; {of SearchOnPath}

procedure TModuleList.saveToDisk(const fname:string);
var
  i :integer;
  FileOut :TextFile;
begin
  assign(FileOut,fname);
  rewrite(FileOut);
  for i := 0 to count-1 do
    with items[i] do
      if State <> msLibrary then
        writeln(FileOut,format('%4d, %6d,"%s","%s"',[CalledByCount,LineCount,
          extractfileName(FullPathName),extractfilePath(FullPathName)]));
  closefile(FileOut);
  showmessage('Saved to '+fname);
end; {of SavetoDisk }

destructor TXrefList.destroy;
begin
  freeandclear;
  inherited destroy;
end;

procedure TXrefList.freeandClear;
var
  i :integer;
begin
  for i:= 0 to count-1 do
    items[i].free;
  clear;
end;
function TXrefList.getXref(index:integer):TXref;
begin
  result := inherited items[index];
end;

procedure TXrefList.setXref(index:integer;Xref:Txref);
begin
  inherited items[index] := Xref;
end;



end.
